/*
 *
 *
 *    mkdir_p.cc
 *
 *    Function to make directory and parents (like mkdir -p).
 *
 *    Copyright (C) 2002 by Matt Roberts, KK5JY,
 *    All rights reserved.
 *
 *    ----------------- L I C E N S E    S T A T E M E N T --------------------
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    ------------------------ E N D    L I C E N S E -------------------------
 *
 *
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "cdir.h"


static void dirpath_trim (char *dir_path) {
	size_t len = strlen(dir_path);
	char *pos  = dir_path + (len - 1);
	
	while ( (pos != dir_path) && (*pos == '/') )
		*pos-- = 0;
}


static char **build_component_list (char *dir_path) {
	char **result = 0, **ri = 0;
	char *start = dir_path;
	char *slash = 0;
	size_t count = 2;
	int absolute = 0;
	size_t i;

	if (*dir_path == '/') {
		absolute = 1;
		++start;
	}

	while ((slash = index(start, '/')) != NULL) {
		++count;
		start = slash + 1;
	}

	result = (char**)malloc(count * sizeof(char*));

	for (i = 0; i < count; ++i)
		result[i] = 0;

	start = dir_path;
	if (absolute)
		++start;

	slash = 0;
	ri = result;
	while ((slash = index(start, '/')) != NULL) {
		*slash = 0;
		*ri++ = start;
		start = slash + 1;
	}

	*ri++ = start;
	*ri = 0;

	return result;
}


int mkdir_p (const char *dir_path, mode_t dir_mode) {
	char *new_path = 0;
	char *med_path = 0;
	char **components = 0;
	char **ci = 0;
	size_t len = 0;
	int result = 0;
	
	/* NULL: if we received a null pointer, bail */
	if (dir_path == NULL) {
		errno = EFAULT;
		result = -1;
		goto cleanup;
	}

	/* LEN: calculate the length of the path */
	len = strlen(dir_path);

	/* EMPTY: if the string is empty, bail */
	if (len == 0) {
		errno = ENOENT;
		result =  -1;
		goto cleanup;
	}

	/* ALLOCATE: */
	new_path = (char*)malloc(len + 1);
	new_path[0] = 0;

	med_path = (char*)malloc(len + 2);
	med_path[0] = 0;

	if ((new_path == NULL) || (med_path == NULL)) {
		errno = ENOMEM;
		result = -1;
		goto cleanup;
	}

	/* COPY: make a working copy of the full path */
	strcpy(new_path, dir_path);

	/* TRIM: clean off trailing slashes */
	dirpath_trim(new_path);
	len = strlen(new_path);

	/* BUILD: build the list of directory components */
	components = build_component_list(new_path);

	if (components == NULL) {
		errno = ENOMEM;
		result = -1;
		goto cleanup;
	}

	for (ci = components; *ci; ++ci) {
		strcat(med_path, *ci);
		if (mkdir(med_path, dir_mode) < 0) {
			result = -1;
			goto cleanup;
		}
		strcat(med_path, "/");
	}

cleanup:
	if (components) free(components);
	if (new_path) free(new_path);
	if (med_path) free(med_path);

	return result;
}

/* end-of-file: mkdir_p.c */
